# -*- coding: utf-8 -*-
import datetime
from django import template
from django.utils.safestring import mark_safe
from django_fiobank.graph import generate_income_issue_graph
from django_fiobank.models import Account

register = template.Library()


@register.simple_tag(takes_context=True)
def render_account_graph(context):
    if 'original' in context and isinstance(context['original'], Account):
        year = datetime.date.today().year
        return mark_safe(generate_income_issue_graph(context['object_id'],
                                                     year))
    else:
        return ''



# -*- coding: utf-8 -*-
import datetime
from django.test import TestCase
from ..models import Account, Transaction


class AccountTest(TestCase):

    def test_account_number_full(self):
        account_number = '42424242'
        bank_code = '2100'

        account = Account(account_number=account_number, bank_code=bank_code)
        expected = '{0}/{1}'.format(account_number, bank_code)

        self.assertEqual(expected, account.account_number_full)

    def test_account_number_full_with_empty_data(self):
        account = Account()
        self.assertEqual('', account.account_number_full)

        account = Account(bank_code='021')
        self.assertEqual('/021', account.account_number_full)

        account = Account(account_number='43242-43')
        self.assertEqual('43242-43/', account.account_number_full)


class TransactionTest(TestCase):

    def test_assigin_data(self):
        data = {
            'transaction_id': 23848928329,
            'date': datetime.date.today(),
            'amount': 666.42,
            'currency': 'CZK',
            'account_number': '058740259847485',
            'account_name': 'Tuhiik',
            'bank_code': '0034982039',
            'bic': '0029384324',
            'bank_name': 'Fio bank',
            'constant_symbol': '0045',
            'variable_symbol': '0123456789',
            'specific_symbol': '0987654321',
            'user_identification': 'Frantisek Vocilka',
            'recipient_message': 'Payment of your invoice',
            'type': 'Bank fee',
            'executor': 'Milos Zenam',
            'specification': 'Bank fee',
            'comment': 'Payment of your invoice',
            'instruction_id': '098765432112345',
            'unsupported_field': 'blah'
        }

        transaction = Transaction()
        transaction.assign(data)

        for property_name, value in data.items():
            if hasattr(transaction, property_name):
                self.assertEqual(value, transaction.__getattribute__(
                    property_name))

    def test_account_full(self):
        account_number = '40293842034'
        bank_code = '0021'
        transaction = Transaction(account_number=account_number,
                                  bank_code=bank_code)

        expected = '{0}/{1}'.format(account_number, bank_code)

        self.assertEqual(expected, transaction.account_full)

    def test_account_number_full_with_empty_data(self):
        transaction = Transaction()
        self.assertEqual('', transaction.account_full)

        transaction = Transaction(bank_code='021')
        self.assertEqual('/021', transaction.account_full)

        transaction = Transaction(account_number='43242-43')
        self.assertEqual('43242-43/', transaction.account_full)



# -*- coding: utf-8 -*-
from django.contrib.sites.models import Site
from django.template import loader
from django.utils.translation import ugettext_lazy as _
from django.core.mail import EmailMultiAlternatives
from django.conf import settings


def token_expired(account):
    """
    Send email to bank account manager

    :param account: Instance of Account model.
    :type account: Account
    """
    site = Site.objects.get_current()
    subject = _('Token expired time validity')
    context = {
        'site_domain': site.domain,
        'site_name': site.name,
        'subject': subject,
        'account_name': account.name,
        'account_number_full': account.account_number_full
    }

    html_content = loader.render_to_string(
        'django_fiobank/emails/token_expired.html', context)

    text_content = loader.render_to_string(
        'django_fiobank/emails/token_expired.txt', context)

    to = account.manager_email
    from_email = settings.DEFAULT_FROM_EMAIL

    msg = EmailMultiAlternatives(subject, text_content, from_email, [to])

    msg.attach_alternative(html_content, 'text/html')
    msg.send()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, primary_key=True, verbose_name='ID')),
                ('name', models.CharField(verbose_name='Name', max_length=255)),
                ('account_number', models.CharField(verbose_name='Account number', max_length=16)),
                ('bank_code', models.CharField(verbose_name='Bank code', max_length=10)),
                ('token', models.CharField(verbose_name='Token', max_length=64)),
                ('token_expire', models.DateField(verbose_name='Token expire', help_text='Date of expiration token.')),
                ('manager_email', models.EmailField(verbose_name='Manager email', max_length=75, help_text='E-mail address bank account manager or owner.')),
            ],
            options={
                'verbose_name': 'Account',
                'verbose_name_plural': 'Accounts',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Transaction',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, primary_key=True, verbose_name='ID')),
                ('transaction_id', models.BigIntegerField(verbose_name='Transaction id')),
                ('date', models.DateField(verbose_name='Date')),
                ('amount', models.FloatField(verbose_name='Amount')),
                ('currency', models.CharField(verbose_name='Currency', max_length=3)),
                ('account_number', models.CharField(verbose_name='Account number', max_length=17)),
                ('account_name', models.CharField(blank=True, null=True, max_length=255, verbose_name='Account name')),
                ('bank_code', models.CharField(verbose_name='Bank code', max_length=10)),
                ('bic', models.CharField(blank=True, null=True, max_length=11, verbose_name='Bic')),
                ('bank_name', models.CharField(verbose_name='Bank name', max_length=255)),
                ('constant_symbol', models.CharField(blank=True, null=True, max_length=4, verbose_name='Constant symbol')),
                ('variable_symbol', models.CharField(blank=True, null=True, max_length=10, verbose_name='Variable symbol')),
                ('specific_symbol', models.CharField(blank=True, null=True, max_length=10, verbose_name='Specific symbol')),
                ('user_identification', models.CharField(blank=True, null=True, max_length=50, verbose_name='User identification')),
                ('recipient_message', models.CharField(blank=True, null=True, max_length=140, verbose_name='Recipient message')),
                ('type', models.CharField(verbose_name='Type', max_length=255)),
                ('executor', models.CharField(blank=True, null=True, max_length=255, verbose_name='Executor')),
                ('specification', models.CharField(blank=True, null=True, max_length=255, verbose_name='Specification')),
                ('comment', models.CharField(blank=True, null=True, max_length=255, verbose_name='Comment')),
                ('instruction_id', models.CharField(verbose_name='Instruction id', max_length=15)),
                ('account', models.ForeignKey(verbose_name='Account', to='django_fiobank.Account')),
            ],
            options={
                'verbose_name': 'Transaction',
                'verbose_name_plural': 'Transactions',
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='transaction',
            unique_together=set([('instruction_id',)]),
        ),
        migrations.AlterUniqueTogether(
            name='account',
            unique_together=set([('account_number', 'bank_code')]),
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
import logging
from requests import HTTPError
import fiobank
from .models import Transaction, Account
from .emails import token_expired

logger = logging.getLogger(__name__)
if not logger.handlers:
    logger.addHandler(logging.StreamHandler())

token_expired_default_sender = token_expired


def download_and_save_bank_transactions():

    today = datetime.date.today()
    account_list = Account.objects.all()
    for account in account_list:

        if account.token_expire < today:
            continue

        try:
            last_transaction = Transaction.objects.get_last_transaction(
                account.id)

            client = fiobank.FioBank(token=account.token)

            if last_transaction:
                lookup_params = {'from_id': last_transaction.transaction_id}
                transaction_list = client.last(**lookup_params)
            else:
                from_date = '{0}-01-01'.format(today.year)
                transaction_list = client.last(from_date=from_date)

        except HTTPError as e:
            logger.error('Error in process account {0}'.format(account))
            logger.exception(e)
            break
        else:
            for data in transaction_list:
                bank_trans = Transaction()
                bank_trans.account = account
                bank_trans.assign(data)
                bank_trans.save()


def check_account_token_time_validity(
        email_sender=token_expired_default_sender):
    """ Check token validity and send emails if expired.

    :param email_sender: Function for send notification email. Default si
    token_expired.
    """
    minimal_validation_date = datetime.date.today() + datetime.timedelta(days=7)
    account_list = Account.objects.all()
    for account in account_list:
        if account.token_expire >= minimal_validation_date:
            email_sender(account)
